#!/bin/bash -e

#Script for building the Oerlikon XScale Development toolchain
#Release 2
#Original code from the arm-linux mailing list
#By Dominic Duval, dduval@oerlikon.ca


PREFIX=${PREFIX:-/tmp/usr/local/arm/}
#KERNEL=${KERNEL:-/home/dduval/XScale_Optimized_packages/linux-2.4.19/}
KERNEL=${KERNEL:-/home/dduval/XScale_Optimized_packages/sources/linux-2.4.19/}
HOST=

# versions of tools to build
BINUTILSVER=${BINUTILS:-2.13.90.0.16}
GCCVER=${GCCVER:-3.2.1}
GLIBCVER=${GLIBCVER:-2.3.1}
GDBVER=${GDBVER:-5.3}

echo "Preparing to build gcc $GCCVER + binutils $BINUTILSVER + glibc $GLIBCVER + gdb $GDBVER, installation directory in $PREFIX, and using kernel headers from $KERNEL."
echo ""
echo "PRESS A KEY TO CONTINUE, CTRL-C to CANCEL"
read x

rm -rf gdb-$GDBVER
tar zxf gdb-$GDBVER.tar.gz
pushd gdb-$GDBVER
./configure --target=arm-linux --build=i686-pc-linux-gnu --prefix=$PREFIX
make
make install
popd
rm -rf gdb-$GDBVER


mkdir -p $PREFIX
if [ "$KERNEL" = "" -o ! -f "$KERNEL/Documentation/arm/README" ]; then
  echo "You haven't changed KERNEL to point at a valid kernel tree"
  exit 1
fi
if [ ! -f binutils-$BINUTILSVER.tar.gz ]; then
  echo Binutils $BINUTILSVER seems to be missing
  exit 1
fi
if [ ! -f gcc-$GCCVER.tar.gz ]; then
  echo GCC $GCCVER seems to be missing
  exit 1
fi
rm -rf binutils-$BINUTILSVER
tar zxf binutils-$BINUTILSVER.tar.gz
patch -p1 < binutils-2.13.90.0.16-oerlikon.patch
pushd binutils-$BINUTILSVER
./configure --target=arm-linux --prefix=$PREFIX --with-cpu=xscale --nfp
make 
make install
popd
rm -rf binutils-$BINUTILSVER
PATH=$PATH:$PREFIX/bin
rm -rf gcc-$GCCVER
tar zxf gcc-$GCCVER.tar.gz
patch -p1 < gcc-3.2.1-oerlikon.patch
pushd gcc-$GCCVER
perl -pi -e 's/^(TARGET_LIBGCC2_CFLAGS.*)/$1 -Dinhibit_libc -D__gthr_posix_h/' gcc/config/arm/t-linux
echo 'T_CFLAGS = -Dinhibit_libc -D__gthr_posix_h' >> gcc/config/arm/t-linux
./configure --target=arm-linux --prefix=$PREFIX $HOST --with-headers=$KERNEL/include --disable-shared --disable-threads --enable-languages="c" --nfp --with-cpu=xscale --without-fp --with-softfloat-support=internal
make 
make install 
popd
rm -rf gcc-$GCCVER

if [ ! -f "$KERNEL/include/linux/version.h" ]; then
  echo "Building glibc requires a configured kernel tree."
  echo "So, build the kernel with the cross compiler in $PREFIX and run this script again."
  exit 1
fi
rm -rf glibc-$GLIBCVER
tar zxf glibc-$GLIBCVER.tar.gz
tar -C glibc-$GLIBCVER -zxf glibc-linuxthreads-$GLIBCVER.tar.gz
pushd glibc-$GLIBCVER
./configure arm-linux --build=i686-pc-linux-gnu --with-headers=$KERNEL/include --enable-add-ons --enable-shared --prefix=$PREFIX --with-cpu=xscale --without-fp --enable-kernel=2.4.18
make
make install
popd
pushd $PREFIX/arm-linux/lib
ln -s $PREFIX/lib/* .
popd
rm -rf glibc-$GLIBCVER
rm -rf gcc-$GCCVER
tar zxf gcc-$GCCVER.tar.gz
patch -p1 < gcc-3.2.1-oerlikon.patch
pushd gcc-$GCCVER
./configure --target=arm-linux --prefix=$PREFIX $HOST --with-headers=$KERNEL/include --with-cpu=xscale --with-softfloat-support=internal --enable-languages=c,c++ --nfp
perl -pi -e 's/int namelen/unsigned int namelen/' libjava/java/net/natInetAddress.cc
make 
make install 
popd
rm -rf gcc-$GCCVER


